/****************************************************************************/
/*																			*/
/*		STK-7125 TvvO											*/
/*			VA													*/
/*																			*/
/*			Copyright   :: At@vWFNg							*/
/*			Cpu         :: SH7125											*/
/*			Language    :: SH-C												*/
/*			File Name   :: sci.c											*/
/*																			*/
/****************************************************************************/
#include "common.h"
#include "7125s.h"
#include "sci.h"

// prototype
void open_sci(sciPara * );

/* Mobt@ */
static char rxb[16];
static char txb[16];
// Mobt@p[^
static const sciBuf rx_prm={ rxb, sizeof(rxb), 0, 0 };
static const sciBuf tx_prm={ txb, sizeof(txb), 0, 0 };
// p[^
const sciPara sci_para = { (sciBuf *)&rx_prm, (sciBuf *)&tx_prm, 38400, 8, 1, sciPrNone, 7 };

/* [NGA */
static sciBuf rec_buf;
static sciBuf send_buf;

/************************************/
/*	SCI						*/
/************************************/
void sci_init(void)
{
	/* 38400,8,1,Nŏ */
	open_sci((sciPara *)&sci_para);
}

/************************************/
/*	SCII[v						*/
/************************************/
void open_sci(sciPara * para)
{
	unsigned char tmp = 0;

	/* [NGA */
	rec_buf.buf   = para->rec->buf;
	rec_buf.size  = para->rec->size;
	rec_buf.wp    = rec_buf.rp  = 0;
	send_buf.buf  = para->send->buf;
	send_buf.size = para->send->size;
	send_buf.wp   = send_buf.rp = 0;

	/* M֎~AM֎~ */
	SCI.SCSCR.BYTE = 0x00;	

	/* ݃x̐ݒ */
	INTC.IPRL.WORD &= ~SCI_IPRMSK;
	INTC.IPRL.WORD |= (para->int_lv << SCI_IPRSFT) & SCI_IPRMSK;

	/* SMRݒ */
	// rbg
	if (para->bit_len == 7) {
		tmp |= 0x40;			// 7bit
	}
	else {
		tmp |= 0x00;			// 8bit
	}
	// peB
	switch(para->parity) {
		case sciPrEven:			// peB
			tmp |= 0x20;
			break;
		case sciPrOdd:			// peB
			tmp |= 0x30;
			break;
		default:				// peB
			tmp |= 0x00;
			break;
	}
	// Xgbvrbg
	if (para->stop_bit == 1) {
		tmp |= 0x00;			// STOP 1
	}
	else {
		tmp |= 0x08;			// STOP 2
	}
	SCI.SCSMR.BYTE = tmp;

	/* BBRݒ */
	tmp = ((PCLK / 32) / para->bps) - 1;
	SCI.SCBRR = tmp;

	/* SSRݒ */
	SCI.SCSSR.BYTE &= ~0x78;		/* Xe[^X(M,G[)NA */
	SCI.SCSCR.BYTE |= 0x70;			/* MAM */
}


/************************************/
/*	SCIN[Y						*/
/************************************/
void close_sci(void)
{
	SCI.SCSCR.BYTE = 0x00;			/* M݋֎~AM֎~ */
	INTC.IPRL.WORD &= ~SCI_IPRMSK;	/* 荞݃xNA */
}


/************************************/
/*	Pi[						*/
/************************************/
short sci_putc(char data)
{
	unsigned long wp;
	short ans;

	wp = send_buf.wp + 1;
	if (wp >= send_buf.size) {
		wp = 0;
	}

	if (wp != send_buf.rp) {
		send_buf.buf[send_buf.wp] = data;
		send_buf.wp = wp;
		SCI.SCSCR.BYTE |= 0x80;		/* M݋ */
		ans = 0;
	}
	else {
		ans = -1;
	}

	return ans;
}


/************************************/
/*	P擾						*/
/************************************/
short sci_getc(void)
{
	unsigned long rp;
	short ans;

	if (rec_buf.rp != rec_buf.wp) {
		ans = (short)rec_buf.buf[rec_buf.rp] & 0x00ff;
		rp = rec_buf.rp + 1;
		if (rp >= rec_buf.size) {
			rp = 0;
		}
		rec_buf.rp = rp;
	}
	else {
		ans = sciEmpty;
	}

	return ans;
}


/************************************/
/*	MG[荞				*/
/************************************/
#pragma interrupt (eri_int)
void eri_int(void)
{
	SCI.SCSSR.BYTE &= ~0x78;		/* G[tONA */
}


/************************************/
/*	M荞					*/
/************************************/
#pragma interrupt (rxi_int)
void rxi_int(void)
{
	unsigned long wp;
	unsigned char data;

	data = SCI.SCRDR;				/* M */
	SCI.SCSSR.BYTE &= ~0x40;		/* RDRFNA */
	
	wp = rec_buf.wp + 1;
	if (wp >= rec_buf.size) {
		wp = 0;
	}
	
	if (wp != rec_buf.rp) {
		rec_buf.buf[rec_buf.wp] = data;
		rec_buf.wp = wp;
	}
}


/************************************/
/*	M荞					*/
/************************************/
#pragma interrupt (txi_int)
void txi_int(void)
{
	unsigned char data;

	if (send_buf.rp != send_buf.wp) {
		data = send_buf.buf[send_buf.rp];
		send_buf.rp++;
		if (send_buf.rp >= send_buf.size) {
			send_buf.rp = 0;
		}

		SCI.SCTDR = data;			/* 1oCgM */
		SCI.SCSSR.BYTE &= ~0x80;	/* TDRE NA */
	}
	else {
		SCI.SCSCR.BYTE &= ~0x80;	/* M݋֎~ */
	}
}


/************************************/
/*	MI					*/
/************************************/
#pragma interrupt (tei_int)
void tei_int(void)
{
	SCI.SCSCR.BYTE &= ~0x04;		/* MI݋֎~ */
}
